# ProbZelus Examples

This project contains the tracker examples presented at the JFLA 2024.

- `boat.zls` simulates the trajectory of a boat with noisy observations
- `kalman.zls` uses a Kalman filter to estimate the trajectory of the boat from noisy observations
- `hmm.zls uses` a probabilistic model to estimate the trajectory of the boat from noisy observations

## Install

You first need to install `probzelus` and `zelus-libs`.
Instructions can be found here: https://github.com/IBM/probzelus

Then, build the examples with:

```bash
$ dune build
```

This command generates one executable `<file>_main.exe` for each file.

Tested with OCaml 5.1.1 

## Boat simulation

The boat simulator prints the stream of positions and the corresponding noisy observations on `stdout`:

```bash
$ ./boat_main.exe
0.000000 -0.417979
0.096252 1.093592
0.188437 1.090253
0.286683 0.121243
0.398736 -2.699409
0.513571 4.050281
0.617300 -0.509879
0.717838 0.018483
0.823656 -1.452874
0.933250 -0.070242
...
^C
```

An easy way to visualize the results is to use [`feedgnuplot`](https://github.com/dkogan/feedgnuplot)

```bash
$ ./boat_main.exe | feedgnuplot --stream 0.05 # Stream stdout, never stops
$ ./boat_main.exe | head -n 100 | feedgnuplot # only show the first 100 steps
```

## Estimate the trajectory

Both `kalman_main.exe` and `hmm_main.exe` read observations on `stdout` to the compute estimated positions.
E.g., using the inputs generated by `boat_main.exe`

```bash
$ ./boat_main.exe | ./kalman_main.exe | head -n 100 | feedgnuplot
$ ./boat_main.exe | ./kalman_main.exe | head -n 100 | feedgnuplot
```

