type 'a distribution = 'a Distribution.t

let bernoulli p = Distribution.bernoulli ~p
let binomial p n = Distribution.binomial ~p ~n
let dirac v = Distribution.dirac ~v
let beta a b = Distribution.beta ~a ~b
let gaussian mu sigma = Distribution.gaussian ~mu ~sigma
let uniform a b = Distribution.uniform ~a ~b
let draw = Distribution.draw
let mean = Distribution.mean

open Ztypes

type prob = { id : int; scores : float array }

let sample (_prob, d) = Distribution.draw d

let observe (prob, d, x) =
  prob.scores.(prob.id) <- prob.scores.(prob.id) +. Distribution.logpdf d x

type 'a infer_state = { mutable particles : 'a array; scores : float array }

let resample alloc copy n state =
  let p_dist =
    Distribution.support ~values:state.particles ~logits:state.scores
  in
  let particles =
    Array.init n (fun i ->
        let m = alloc () in
        let p = Distribution.draw p_dist in
        copy p m;
        m)
  in
  state.particles <- particles

let infer_pf n (Cnode { alloc; reset; step; copy }) =
  let infer_alloc () =
    { particles = Array.init n (fun _ -> alloc ()); scores = Array.make n 0. }
  in
  let infer_reset state =
    Array.iter reset state.particles;
    Array.fill state.scores 0 n 0.
  in
  let infer_step state data =
    let values =
      Array.mapi
        (fun i s -> step s ({ id = i; scores = state.scores }, data))
        state.particles
    in
    let v_dist = Distribution.support ~values ~logits:state.scores in
    resample alloc copy n state;
    Array.fill state.scores 0 n 0.;
    v_dist
  in
  let infer_copy _ _ = () in
  Cnode
    {
      alloc = infer_alloc;
      reset = infer_reset;
      step = infer_step;
      copy = infer_copy;
    }
